package aprioriUtil;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;
public class AssocRule<I> {

    private final Set<I> ante = new HashSet<>();
    private final Set<I> cons = new HashSet<>();
    private double conf;

    public AssocRule(Set<I> ante,
            Set<I> cons,
            double conf) {
        Objects.requireNonNull(ante, "Antecedent value in the rule is null.");
        Objects.requireNonNull(cons, "Consequent value in the rule is null.");
        this.ante.addAll(ante);
        this.cons.addAll(cons);
        this.conf = conf;
    }

    public AssocRule(Set<I> ante, Set<I> cons) {
        this(ante, cons, Double.NaN);
    }

    public Set<I> getAnte() {
        return Collections.<I>unmodifiableSet(ante);
    }

    public Set<I> getCons() {
        return Collections.<I>unmodifiableSet(cons);
    }

    public double getConf() {
        return conf;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(Arrays.toString(ante.toArray()));
        sb.append(" -> ");
        sb.append(Arrays.toString(cons.toArray()));
        sb.append(": "+conf);
        return sb.toString();
    }

    @Override
    public int hashCode() {
        return ante.hashCode() ^ cons.hashCode();
    }

    @Override
    public boolean equals(Object obj) {
        AssocRule<I> other = (AssocRule<I>) obj;

        return ante.equals(other.ante) && cons.equals(other.cons);
    }
}