package main;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.FileReader;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.text.DefaultCaret;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.*;
import java.util.regex.Pattern;
import aprioriUtil.*;



public class MainDriver {
    private static String Generate = "Generate";
    static JButton generateButton;
    public static JTextArea resultArea;
    public static JComboBox<String> hasHeaderCombo;
    public static JTextField inputPath;
    public static JTextField minSup;
    public static JTextField minConf;
    static JFrame frame;
    static JPanel panel;

    public static void main(String[] args) throws IOException {
        inputPath = new JTextField(45);
        JLabel inputPathLabel = new JLabel("Input File Path:");
        inputPath.setToolTipText("Input File Path");

        JLabel supLabel = new JLabel("Min Support:");
        minSup = new JTextField(3);
        minSup.setText("0.1");
        minSup.setToolTipText("Minimum support should range between 0 and 1.");

        JLabel confLabel = new JLabel("Min Confidence:");
        minConf = new JTextField(3);
        minConf.setText("0.1");
        minConf.setToolTipText("Minimum confidence should range between 0 and 1.");

        generateButton = new JButton("Generate Rules");
        generateButton.setActionCommand(Generate);

        AprioriAlgorithm aprioriInvoker = new AprioriAlgorithm();
        generateButton.addActionListener(aprioriInvoker);
        inputPath.setActionCommand(Generate);
        inputPath.addActionListener(aprioriInvoker);
        inputPath.setText("enter the path of the input file");
//        inputPath.requestFocus();

        resultArea = new JTextArea(15, 100);
        resultArea.setEditable(false);
        resultArea.setWrapStyleWord(true);

        DefaultCaret caret = (DefaultCaret) resultArea.getCaret();
        caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);

        // starting with input text field and its label this is to read the text file path and followed by two input fields
        // which takes two inputs min confidence and min support and their labels
        // Now we design a button which invokes the generation of apriori rules and its support score for each rule.

        JPanel inputPanel = new JPanel();
        inputPanel.setLayout(new FlowLayout());
        inputPanel.add(inputPathLabel);
        inputPanel.add(inputPath);
        inputPanel.add(supLabel);
        inputPanel.add(minSup);
        inputPanel.add(confLabel);
        inputPanel.add(minConf);

        JPanel inputPanel_1 = new JPanel();
        inputPanel_1.setLayout(new FlowLayout());
        inputPanel_1.add(generateButton);
        JScrollPane scroller = new JScrollPane(resultArea);
        scroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

        panel = new JPanel();
        panel.setOpaque(true);
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.add(inputPanel);
        panel.add(inputPanel_1);
        panel.add(scroller);

        frame = new JFrame("Apriori Algorithm");
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.getContentPane().add(BorderLayout.CENTER, panel);
        frame.pack();
        frame.setLocationByPlatform(true);
        frame.setVisible(true);
        frame.setResizable(true);

        //This code is concentrated on building the layout which is frame plus panel, and labeling it with a text AA and enabling the ability to scroll up and down
    }

    public static class AprioriAlgorithm implements ActionListener {
        private boolean validateConfOrSup(String value) {
            try {
                float parsedVal = Float.parseFloat(value);
                if ((parsedVal > 1) || (parsedVal < 0)) {
                    return false;
                }
            } catch (Exception e) {
                return false;
            }
            return true;
        }

        private String checkParameters() {
            if (inputPath.getText().trim().equals("")) {
                return "Input path cannot be empty";
            }
            if (!validateConfOrSup(minSup.getText().trim())) {
                return "Minimum support value is not valid";
            }
            if (!validateConfOrSup(minConf.getText().trim())) {
                return "Minimum confidence value id not valid";
            }
            return "Valid";
        }

        public void actionPerformed(final ActionEvent ae) {
            resultArea.setText("");
            String validationResult = checkParameters();
            if (validationResult.equals("Valid")) {
                String command = ae.getActionCommand();
                if (Generate.equals(command)) {
                    try {
                        float conf = Float.parseFloat(minConf.getText());
                        float sup = Float.parseFloat(minSup.getText());
                        String inputPathOfTrasaction = inputPath.getText().toString();
                        String result = runAprioriAlgo(sup, conf, inputPathOfTrasaction);
                        resultArea.append("\n\n" + result);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            } else {
                resultArea.append("Error:- " + validationResult);
            }

        }
    }

    public void actionTrigger(final ActionEvent ev) {
        try {
            float conf = Float.parseFloat(minConf.getText());
            float sup = Float.parseFloat(minSup.getText());
            String inputPathOfTransaction = inputPath.toString();
            runAprioriAlgo(sup, conf,inputPathOfTransaction);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static String runAprioriAlgo(float sup, float conf,String inputPathOfTrasactions) throws IOException {
        List<Set<String>> itemsetList = new ArrayList<>();
        AprioriFrequentItemsetGenerator<String> freqItemsetGenerator = new AprioriFrequentItemsetGenerator<>();

        String currentPath = new java.io.File(".").getCanonicalPath();


        currentPath = currentPath.replace('\\','/');

        System.out.println("Current dir:" + currentPath);

        String outFileName = currentPath+"/Reports/result.txt";
        File outFile = new File(outFileName);
        //C:/Users/Mohan B/Downloads/AssociationRulesExampleCode/AssociationRulesExampleCode/transactions.txt
        outFile.getParentFile().mkdirs();
        PrintWriter resultPW = new PrintWriter(outFile);
        String line;
        int i = 1;
        System.out.println(inputPathOfTrasactions.toString());
        FileReader fr = new FileReader(inputPathOfTrasactions);
        BufferedReader br = new BufferedReader(fr);
        while ((line = br.readLine()) != null) {
            line = line.trim();
            String arr[] = line.split(Pattern.quote(" "));
            HashSet<String> utilSet = new HashSet<String>();
            for (String s : arr) {
                utilSet.add(s);
            }
            itemsetList.add(utilSet);
        }

        long startTime = System.nanoTime();
        FreqItemSet<String> data = freqItemsetGenerator.generate(itemsetList, sup);
        long endTime = System.nanoTime();



        resultPW.println("Frequent itemsets : ");
        for (Set<String> itemset : data.getFreqItemList()) {
            resultPW.printf("%2d: %9s, support: %1.1f\n", i++, itemset, data.getSup(itemset));
        }

        System.out.printf("Mined frequent itemsets in %d milliseconds.\n",
                (endTime - startTime) / 1_000_000);



        startTime = System.nanoTime();
        List<AssocRule<String>> associationRuleList = new AssociationRuleGenerator<String>()
                .mineAssociationRules(data, conf);
        endTime = System.nanoTime();

        i = 1;

        resultPW.println();
        resultPW.println("Association Rules: ");

        for (AssocRule<String> rule : associationRuleList) {
            resultPW.printf("%2d) %s\n", i++, rule);
        }

        System.out.printf("Mined association rules in %d milliseconds.\n",
                (endTime - startTime) / 1_000_000);
        resultPW.flush();
        resultPW.close();
        return "Output has been written to filepath : " + outFileName;
    }

}


